package com.supwisdom.dlpay.framework.core;

import org.jose4j.jwk.JsonWebKey;
import org.jose4j.jws.AlgorithmIdentifiers;
import org.jose4j.jws.JsonWebSignature;
import org.jose4j.jwt.JwtClaims;
import org.jose4j.lang.JoseException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Configuration
@EnableAutoConfiguration
@Component
public class JwtTokenUtil {
  @Value("${jwt.secret}")
  private String secret;
  @Value("${jwt.expiration}")
  private Long expiration = 3600L;
  @Value("${jwt.header}")
  private String header = "supwisdom";

  public String generateToken(Map<String, Object> params) throws JoseException {
    JwtClaims claims = new JwtClaims();
    claims.setIssuer(params.get("issuer").toString());  // who creates the token and signs it
    claims.setExpirationTimeMinutesInTheFuture(expiration / 60); // time when the token will expire (10 minutes from now)
    claims.setGeneratedJwtId(); // a unique identifier for the token
    claims.setIssuedAtToNow();  // when the token was issued/created (now)
    claims.setNotBeforeMinutesInThePast(2); // time before which the token is not yet valid (2 minutes ago)
    claims.setSubject(params.get("subject").toString()); // the subject/principal is whom the token is about
    /*
    claims.setClaim("email", "mail@example.com"); // additional claims/attributes about the subject can be added
    List<String> groups = Arrays.asList("group-one", "other-group", "group-three");
    claims.setStringListClaim("groups", groups); // multi-valued claims work too and will end up as a JSON array
     */

    Map<String, Object> keySpec = new HashMap<>();
    keySpec.put("kty", "oct");
    keySpec.put("k", secret);
    JsonWebKey key = JsonWebKey.Factory.newJwk(keySpec);
    JsonWebSignature jws = new JsonWebSignature();
    jws.setPayload(claims.toJson());
    jws.setKey(key.getKey());
    jws.setKeyIdHeaderValue(key.getKeyId());
    jws.setAlgorithmHeaderValue(AlgorithmIdentifiers.HMAC_SHA256);
    return jws.getCompactSerialization();
  }

  public String generateToken(UserDetails userDetails) throws JoseException {
    Map<String, Object> claims = new HashMap<>();
    claims.put("uid", userDetails.getUsername());
    return generateToken(claims);
  }
}
