package com.supwisdom.dlpay

import com.supwisdom.dlpay.framework.filter.ValidateCodeFilter
import com.supwisdom.dlpay.framework.security.MyAuthenticationFailureHandler
import com.supwisdom.dlpay.framework.security.MyAuthenticationSuccessHandler
import io.lettuce.core.ReadFrom
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.autoconfigure.SpringBootApplication
import org.springframework.boot.runApplication
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.core.annotation.Order
import org.springframework.data.redis.connection.RedisConnectionFactory
import org.springframework.data.redis.connection.RedisPassword
import org.springframework.data.redis.connection.RedisStandaloneConfiguration
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory
import org.springframework.data.redis.core.RedisKeyValueTemplate
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.data.redis.repository.configuration.EnableRedisRepositories
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter
import org.springframework.security.config.http.SessionCreationPolicy
import org.springframework.security.core.userdetails.User
import org.springframework.security.core.userdetails.UserDetailsService
import org.springframework.security.provisioning.InMemoryUserDetailsManager
import org.springframework.security.web.authentication.AuthenticationFailureHandler
import org.springframework.security.web.authentication.logout.CookieClearingLogoutHandler
import javax.servlet.Filter


@Configuration
@EnableRedisRepositories
class AppConfig {

    @Value("\${redis.server}")
    lateinit var server: String

    @Value("\${redis.port}")
    var port: Int = 0

    @Value("\${redis.database}")
    var database: Int = 0

    @Value("\${redis.password}")
    lateinit var password: String

    @Bean
    fun redisConnectionFactory(): RedisConnectionFactory {
        val clientConfig = LettuceClientConfiguration.builder()
                .readFrom(ReadFrom.SLAVE_PREFERRED)
                .build()
        val serverConfig = RedisStandaloneConfiguration(server, port)
        if (!password.isNullOrEmpty()) {
            serverConfig.password = RedisPassword.of(password)
        }
        serverConfig.database = database

        return LettuceConnectionFactory(serverConfig, clientConfig)
    }
}

@EnableWebSecurity
class WebSecurityConfig {
    @Bean
    fun userDetailsService(): UserDetailsService {
        val manager = InMemoryUserDetailsManager()
        manager.createUser(User.withDefaultPasswordEncoder()
                .username("admin")
                .password("123456")
                .roles("USER").build())
        return manager
    }

    companion object {
        @Configuration
        @Order(1)
        class ApiWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {
//            @Autowired
//            private lateinit var clientRegistrationRepository: ClientRegistrationRepository

            override fun configure(http: HttpSecurity) {
                http.authorizeRequests()
//                        .antMatchers("/login", "/resources/**", "/about", "/common/**").permitAll()
//                        .antMatchers("/admin/**").hasRole("ADMIN")
//                        .antMatchers("/db/**").access("hasRole('ADMIN') and hasRole('DBA')")
                        .antMatchers("/**").permitAll()
                        .antMatchers("/admin/**").hasRole("ADMIN")
                        .anyRequest().authenticated()
                        .and().httpBasic()
                        .and()
                        .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
//                        .oauth2Client()
//                        .clientRegistrationRepository(clientRegistrationRepository)
//                        .authorizedClientRepository(this.authorizedClientRepository())
//                        .authorizedClientService(this.authorizedClientService())
//                        .authorizationCodeGrant()
//                        .authorizationRequestRepository(this.authorizationRequestRepository())
//                        .authorizationRequestResolver(this.authorizationRequestResolver())
//                        .accessTokenResponseClient(this.accessTokenResponseClient())
            }
        }


        @Configuration
        class MvcWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {
            override fun configure(http: HttpSecurity) {
                http.authorizeRequests()
                        .anyRequest().authenticated()
                        .and()
                        .formLogin()
                        .loginPage("/login")
                        .loginProcessingUrl("/login/form")
                        .and()
                        .logout()
                        .logoutSuccessUrl("/login")
                        .invalidateHttpSession(true)
                        .addLogoutHandler(CookieClearingLogoutHandler())
            }
        }
    }
}


@SpringBootApplication
class DlpayApplication

fun main(args: Array<String>) {
    runApplication<DlpayApplication>(*args)
}
